@@ -15,7 +15,7 @@
  * 3. Neither the name of the author nor the names of its contributors
  *    may be used to endorse or promote products derived from this
  *    software without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
@@ -48,6 +48,9 @@
 #include <machine/apm_bios.h>
 #include <kvm.h>
 #include <errno.h>
+#include <sys/socket.h>
+#include <net/if.h>
+#include <net/if_mib.h>
 
 #include "main.h"
 #include "machine.h"
@@ -146,11 +149,11 @@ int machine_get_fs(mounts_type fs[], int *cnt)
 {
 	struct statfs *mntbuf;
 	struct statfs *pp;
-	int statcnt, fscnt, i; 
+	int statcnt, fscnt, i;
 
 	fscnt = getmntinfo(&mntbuf, MNT_WAIT);
-	if (fscnt == 0) 
-	{ 
+	if (fscnt == 0)
+	{
 		perror("getmntinfo");
 		return(FALSE);
 	}
@@ -182,7 +185,7 @@ int machine_get_fs(mounts_type fs[], int *cnt)
 			statcnt++;
 		}
 	}
-	
+
 	*cnt = statcnt;
 
 	return(TRUE);
@@ -295,7 +298,7 @@ int machine_get_procs(LinkedList *procs)
 
 	kprocs = kvm_getprocs(kvmd, KERN_PROC_ALL, 0, &nproc);
 	if (kprocs == NULL)
-	{   
+	{
 		perror("kvm_getprocs");
 		kvm_close(kvmd);
 		return(FALSE);
@@ -382,7 +385,7 @@ int machine_get_uptime(double *up, double *idle)
 	else
 		*idle = 100.*curr_load.idle/curr_load.total;
 
-	return(TRUE); 
+	return(TRUE);
 }
 
 static int swapmode(int *retavail, int *retfree)
@@ -416,4 +419,63 @@ static int swapmode(int *retavail, int *retfree)
 	return(n);
 }
 
+/*************************************************************************
+ * Read interface statistics from system and  store in the struct 
+ * passed as a pointer. If there are no errors, it returns 1. If errors, 
+ * returns 0.
+ ************************************************************************* 
+ */
+int machine_get_iface_stats (IfaceInfo *interface)
+{
+	static int      first_time = 1;	/* is it first time we call this function? */
+	int             rows;
+	int             name[6] = {CTL_NET, PF_LINK, NETLINK_GENERIC, IFMIB_IFDATA, 0, IFDATA_GENERAL};
+	size_t          len;
+	struct ifmibdata ifmd; /* ifmibdata contains the network statistics */
+
+	len = sizeof(rows);
+	/* get number of interfaces */
+	if (sysctlbyname("net.link.generic.system.ifcount", &rows, &len, NULL, 0) == 0) {
+		interface->status = down; /* set status down by default */
+
+		len = sizeof(ifmd);
+		/* walk through all interfaces in the ifmib table from last to first */
+		for ( ; rows > 0; rows--) {
+			name[4] = rows; /* set the interface index */
+			/* retrive the ifmibdata for the current index */
+			if (sysctl(name, 6, &ifmd, &len, NULL, 0) == -1) {
+				perror("read sysctl");
+				break;
+			}
+			/* check if its interface name matches */
+			if (strcmp(ifmd.ifmd_name, interface->name) == 0) {
+				interface->last_online = time(NULL);	/* save actual time */
+
+				if ((ifmd.ifmd_flags & IFF_UP) == IFF_UP)
+					interface->status = up;	/* is up */
+
+				interface->rc_byte = ifmd.ifmd_data.ifi_ibytes;
+				interface->tr_byte = ifmd.ifmd_data.ifi_obytes;
+				interface->rc_pkt = ifmd.ifmd_data.ifi_ipackets;
+				interface->tr_pkt = ifmd.ifmd_data.ifi_opackets;
+
+				if (first_time) {
+					interface->rc_byte_old = interface->rc_byte;
+					interface->tr_byte_old = interface->tr_byte;
+					interface->rc_pkt_old = interface->rc_pkt;
+					interface->tr_pkt_old = interface->tr_pkt;
+					first_time = 0;	/* now it isn't first time */
+				}
+				return 1;
+			}
+		}
+		/* if we are here there is no interface with the given name */
+		return 0;
+	} else {
+		perror("read sysctlbyname");
+		return 0;
+	}
+} /* get_iface_stats() */
+
+
 #endif /* __FreeBSD__ */
