@@ -7,9 +7,11 @@
 #include <fcntl.h>
 #include <termios.h>
 #include <errno.h>
+#include <locale.h>
 #include <signal.h>
 #include <netdb.h>
 #include <ctype.h>
+#include <sys/stat.h>
 #include <sys/utsname.h>
 #include <sys/param.h>
 
@@ -22,6 +24,7 @@
 #include "shared/sockets.h"
 #include "shared/debug.h"
 #include "shared/report.h"
+#include "shared/configfile.h"
 
 #include "batt.h"
 #include "chrono.h"
@@ -31,11 +34,15 @@
 #include "load.h"
 #include "mem.h"
 #include "machine.h"
+#include "iface.h"
+#ifdef LCDPROC_EYEBOXONE
+# include "eyebox.h"
+#endif
 
 // TODO: Commenting...  Everything!
 
 int Quit = 0;
-int sock;
+int sock = -1;
 
 char *version = VERSION;
 char *build_date = __DATE__;
@@ -54,53 +61,78 @@ static struct utsname unamebuf;
 static void HelpScreen(int exit_state);
 static void exit_program(int val);
 static void main_loop();
+static int process_configfile(char *cfgfile);
 
-static int islow = -1;
 
 // 1/8th second is a single time unit...
-#define TIME_UNIT 125000
+#define TIME_UNIT	125000
+
+#if !defined(SYSCONFDIR)
+# define SYSCONFDIR	"/etc"
+#endif
+
+#define UNSET_INT		-1
+#define UNSET_STR		"\01"
+#define DEFAULT_SERVER		"127.0.0.1"
+#define DEFAULT_CONFIGFILE	SYSCONFDIR "/lcdproc.conf"
+#define DEFAULT_REPORTDEST	RPT_DEST_STDERR
+#define DEFAULT_REPORTLEVEL	RPT_WARNING
 
 /* list of modes to run */
 mode sequence[] =
 {
 	// flags default ACTIVE will run by default
-	// longname    which on  off inv  timer   flags  
-	{ "CPU",       'C',   1,    2, 0, 0xffff, ACTIVE, cpu_screen },    // [C]PU
-	{ "Memory",    'M',   4,   16, 0, 0xffff, ACTIVE, mem_screen },    // [M]emory
-	{ "Load",      'L',  64,  128, 1, 0xffff, ACTIVE, xload_screen },  // [L]oad (load histogram)
-	{ "TimeDate",  'T',   4,   64, 0, 0xffff, ACTIVE, time_screen },   // [T]ime/Date
-	{ "About",     'A', 999, 9999, 0, 0xffff, ACTIVE, credit_screen }, // [A]bout (credits)
-	{ "SMP-CPU",   'P',   1,    2, 0, 0xffff, 0, cpu_smp_screen },     // CPU_SM[P]
-	{ "OldTime",   'O',   4,   64, 0, 0xffff, 0, clock_screen },       // [O]ld Timescreen
-	{ "BigClock",  'K',   4,   64, 0, 0xffff, 0, big_clock_screen },   // big cloc[K] 
-	{ "Uptime",    'U',   4,  128, 0, 0xffff, 0, uptime_screen },      // Old [U]ptime Screen
-	{ "Battery",   'B',  32,  256, 1, 0xffff, 0, battery_screen },     // [B]attery Status
-	{ "CPUGraph",  'G',   1,    2, 0, 0xffff, 0, cpu_graph_screen },   // CPU histogram [G]raph
-	{ "ProcSize",  'S',  16,  256, 1, 0xffff, 0, mem_top_screen },     // [S]ize of biggest processes
-	{ "Disk",      'D', 256,  256, 1, 0xffff, 0, disk_screen },        // [D]isk stats
-	{ "MiniClock", 'N',   4,   64, 0, 0xffff, 0, mini_clock_screen },  // Mi[n]i clock 
-	{  NULL, 0, 0, 0, 0, 0,},		// No more..  all done.
+	// longname    which on  off inv  timer   flags
+	{ "CPU",       'C',   1,    2, 0, 0xffff, ACTIVE, cpu_screen        },	// [C]PU
+	{ "Iface",     'I',   1,    2, 0, 0xffff, 0,      iface_screen      }, 	// [I]face
+	{ "Memory",    'M',   4,   16, 0, 0xffff, ACTIVE, mem_screen        },	// [M]emory
+	{ "Load",      'L',  64,  128, 1, 0xffff, ACTIVE, xload_screen      },	// [L]oad (load histogram)
+	{ "TimeDate",  'T',   4,   64, 0, 0xffff, ACTIVE, time_screen       },	// [T]ime/Date
+	{ "About",     'A', 999, 9999, 0, 0xffff, ACTIVE, credit_screen     },	// [A]bout (credits)
+	{ "SMP-CPU",   'P',   1,    2, 0, 0xffff, 0,      cpu_smp_screen    },	// CPU_SM[P]
+	{ "OldTime",   'O',   4,   64, 0, 0xffff, 0,      clock_screen      },	// [O]ld Timescreen
+	{ "BigClock",  'K',   4,   64, 0, 0xffff, 0,      big_clock_screen  },	// big cloc[K]
+	{ "Uptime",    'U',   4,  128, 0, 0xffff, 0,      uptime_screen     },	// Old [U]ptime Screen
+	{ "Battery",   'B',  32,  256, 1, 0xffff, 0,      battery_screen    },	// [B]attery Status
+	{ "CPUGraph",  'G',   1,    2, 0, 0xffff, 0,      cpu_graph_screen  },	// CPU histogram [G]raph
+	{ "ProcSize",  'S',  16,  256, 1, 0xffff, 0,      mem_top_screen    },	// [S]ize of biggest processes
+	{ "Disk",      'D', 256,  256, 1, 0xffff, 0,      disk_screen       },	// [D]isk stats
+	{ "MiniClock", 'N',   4,   64, 0, 0xffff, 0,      mini_clock_screen },	// Mi[n]i clock
+	{  NULL, 0, 0, 0, 0, 0, 0, NULL},			  	// No more..  all done.
 };
 
 
 
-// TODO: Config file; not just command line
+/* All variables are set to 'unset' values*/
+static int islow = -1;
+char *progname = "lcdproc";
+char *server 	= NULL;
+int port	= LCDPORT;
+int foreground 	= UNSET_INT;
+static int report_level = UNSET_INT;
+static int report_dest 	= UNSET_INT;
+char *configfile = NULL;
+char *displayname = NULL;
+
 
 const char *get_hostname()
 {
 	return(unamebuf.nodename);
 }
 
+
 const char *get_sysname()
 {
 	return(unamebuf.sysname);
 }
 
+
 const char *get_sysrelease()
 {
 	return(unamebuf.release);
 }
 
+
 int set_mode(int shortname, char *longname, int state)
 {
 	int k;
@@ -108,13 +140,15 @@ int set_mode(int shortname, char *longname, int state)
 	/* ignore already selected modes */
 	for (k = 0; sequence[k].which != 0; k++) {
 		if (((sequence[k].longname != NULL) &&
-		     (0 == strcasecmp(longname, sequence[k].longname))) || 
+		     (0 == strcasecmp(longname, sequence[k].longname))) ||
 		    (toupper(shortname) == sequence[k].which)) {
 			if (!state) {
-				/* clean both the active and inititialized bits since we delete the screen */
+				/* clean both the active and initialized bits since we delete the screen */
 				sequence[k].flags &= (~ACTIVE & ~INITIALIZED);
-				sprintf (buffer, "screen_del %c\n", shortname );
-				sock_send_string (sock, buffer);
+				/* delete the screen if we are connected */
+				if (sock >= 0) {
+					sock_printf(sock, "screen_del %c\n", sequence[k].which);
+				}	
 			} else
 				sequence[k].flags |= ACTIVE;
 			return 1; //found
@@ -123,23 +157,26 @@ int set_mode(int shortname, char *longname, int state)
 	return 0; //not found
 }
 
+
 void clear_modes()
 {
 	int k;
+
 	/* ignore already selected modes */
 	for (k = 0; sequence[k].which != 0; k++) {
 		sequence[k].flags &= (~ACTIVE);
 	}
 }
 
+
 int
 main(int argc, char **argv)
 {
-	int i;
+	int cfgresult;
 	int c;
-	char *server = NULL;
-	int port = LCDPORT;
-	int daemonize = FALSE;
+
+	/* set locale for cwdate & time formatting in chrono.c */
+	setlocale(LC_TIME, "" );
 
 	/* get uname information */
 	if (uname(&unamebuf) == -1) {
@@ -156,29 +193,37 @@ main(int argc, char **argv)
 	/* No error output from getopt */
 	opterr = 0;
 
-	/* get options */
-	while ((c = getopt( argc, argv, "s:p:e:dhv")) > 0) {
+	/* get options from command line */
+	while ((c = getopt( argc, argv, "s:p:e:c:fhv")) > 0) {
+		char *end;
+
 		switch (c) {
+			// c is for config file
+			case 'c':
+				configfile = optarg;
+				break;
 			// s is for server
 			case 's':
-				if (server == NULL)
-					server = optarg;
-				else
-					fprintf(stderr, "Ignoring additional server: %s\n", optarg);
+				server = optarg;
 				break;
 			// p is for port
 			case 'p':
-				port = atoi(optarg);
-				if ((port < 1) && (port > 0xFFFF)) {
-					fprintf(stderr, "Warning:  Port %d outside of legal range\n", port);
-					return(EXIT_FAILURE);
-				}	
+				port = strtol(optarg, &end, 0);
+				if ((*optarg == '\0') || (*end != '\0') ||
+				    (port <= 0) || (port >= 0xFFFF)) {
+					fprintf(stderr, "Illegal port value %s\n", optarg);
+					exit(EXIT_FAILURE);
+				}
 				break;
 			case 'e':
-				islow = atoi(optarg);
+				islow = strtol(optarg, &end, 0);
+				if ((*optarg == '\0') || (*end != '\0') || (islow < 0)) {
+					fprintf(stderr, "Illegal delay value %s\n", optarg);
+					exit(EXIT_FAILURE);
+				}
 				break;
-			case 'd':
-				daemonize = TRUE;
+			case 'f':
+				foreground = TRUE;
 				break;
 			case 'h':
 				HelpScreen(EXIT_SUCCESS);
@@ -195,27 +240,52 @@ main(int argc, char **argv)
 		}
 	}	
 
-	/* parse arguments */
-	if (argc > optind)		//user specified some modes, so clear all defaults
-		clear_modes();
-	
-	for (i = max(optind, 1); i < argc; i++) {
-		int shortname = (strlen(argv[i]) == 1) ? toupper(argv[i][0]) : '\0';
-		int found = set_mode(shortname, argv[i], 1);
-		
-		if (!found) {
-			fprintf(stderr, "Invalid Mode: %c, ignoring\n", argv[i][0]);
-			return(EXIT_FAILURE);
-		}
+	/* Read config file*/
+	cfgresult = process_configfile(configfile);
+	if (cfgresult < 0) {
+		fprintf(stderr, "Error reading config file");
+		exit(EXIT_FAILURE);
+	}	
+
+	/* Set default reporting options */
+	if (report_dest == UNSET_INT)
+		report_dest = DEFAULT_REPORTDEST;
+	if( report_level == UNSET_INT)
+		report_level = DEFAULT_REPORTLEVEL;
+
+	/* Set reporting settings */
+	set_reporting("lcdproc", report_level, report_dest);
+
+	/* parse non-option arguments: modes to add/delete */
+	if (argc > max(optind, 1)) {
+		int i;
+
+		// if no config file was read, ignore hard coded default modes
+		if (cfgresult == 0)
+			clear_modes();
+
+		// turn additional options on or off (using ! as prefix)
+		for (i = max(optind, 1); i < argc; i++) {
+			int state = (*argv[i] == '!') ? 0 : 1;
+			char *name = (state) ? argv[i] : argv[i]+1;
+			int shortname = (strlen(name) == 1) ? name[0] : '\0';
+			// debug: fprintf(stderr, "%s%s\n", (state) ? "" : "!", name);
+			int found = set_mode(shortname, name, state);
+
+			if (!found) {
+				fprintf(stderr, "Invalid Screen: %s\n", name);
+				return(EXIT_FAILURE);
+			}
+		}	
 	}
 
 	if (server == NULL)
-		server = "localhost";
+		server = DEFAULT_SERVER;
 
 	// Connect to the server...
 	usleep(500000);		// wait for the server to start up
 	sock = sock_connect(server, port);
-	if (sock <= 0) {
+	if (sock < 0) {
 		fprintf(stderr, "Error connecting to LCD server %s on port %d.\n"
 		                "Check to see that the server is running and operating normally.\n",
 				server, port);
@@ -231,7 +301,7 @@ main(int argc, char **argv)
 	lcd_cellwid = 5;
 	lcd_cellhgt = 8;
 
-	if (daemonize) {
+	if (foreground == 0) {
 		if (daemon(1,0) != 0) {
 			fprintf(stderr, "Error: daemonize failed");
 			return(EXIT_FAILURE);
@@ -250,37 +320,121 @@ main(int argc, char **argv)
 	return(0);
 }
 
+
+/* reads and parses configuration filei
+ * returns:  1 if configfile was read,
+ *           0 if default configfile doesn't exist
+ *          <0 on error
+ */
+static int
+process_configfile(char *configfile)
+{
+	int k;
+	const char *tmp;
+
+	debug(RPT_DEBUG, "%s(%s)", __FUNCTION__, (configfile) ? configfile : "<null>");
+
+	/* Read config settings*/
+
+	if (configfile == NULL) {
+		struct stat statbuf;
+
+		// if default config file does not exist, do not consider this an error
+		if ((lstat(DEFAULT_CONFIGFILE, &statbuf) == -1) && (errno = ENOENT))
+			return 0;
+
+		configfile = DEFAULT_CONFIGFILE;
+	}
+		
+	if (config_read_file(configfile) != 0) {
+		report(RPT_CRIT, "Could not read config file: %s", configfile);
+		return -1;
+		//report(RPT_WARNING, "Could not read config file: %s", configfile);
+	}
+
+	if (server == NULL) {
+		server = strdup(config_get_string(progname, "Server", 0, DEFAULT_SERVER));
+	}
+	if (port == UNSET_INT) {
+		port = config_get_int(progname, "Port", 0, LCDPORT);
+	}
+
+	if(report_level == UNSET_INT) {
+		report_level = config_get_int(progname, "ReportLevel", 0, RPT_WARNING);
+	}
+	if(report_dest == UNSET_INT) {
+		if (config_get_bool(progname, "ReportToSyslog", 0, 0)) {
+			report_dest = RPT_DEST_SYSLOG;
+		} else {
+			report_dest = RPT_DEST_STDERR;
+		}
+	}
+	if (foreground == UNSET_INT) {
+		foreground = config_get_bool(progname, "Foreground", 0, 0);
+	}
+	if (islow < 0) {
+		islow = config_get_int(progname, "Delay", 0, -1);
+	}
+
+	if ((tmp = config_get_string(progname, "DisplayName", 0, NULL)) != NULL)
+		displayname = strdup(tmp);
+
+	/*
+	 * check for config file variables to override all the sequence defaults
+	 */
+	for (k = 0; sequence[k].which  != 0; k++) {
+		if (sequence[k].longname != NULL) {
+			sequence[k].on_time = config_get_int(sequence[k].longname, "OnTime", 0, sequence[k].on_time);
+			sequence[k].off_time = config_get_int(sequence[k].longname, "OffTime", 0, sequence[k].off_time);
+			sequence[k].show_invisible = config_get_bool(sequence[k].longname, "ShowInvisible", 0, sequence[k].show_invisible);
+			if (config_get_bool( sequence[k].longname, "Active", 0, sequence[k].flags & ACTIVE))
+				sequence[k].flags |= ACTIVE;
+			else
+				sequence[k].flags &= (~ACTIVE);
+		}
+	}
+
+	return 1;
+}
+
 void
 HelpScreen (int exit_state)
 {
 	fprintf(stderr,
-		"Usage: lcdproc [<options>] [<modes> ...]\n"
+		"lcdproc - LCDproc system status information viewer\n"
+		"\n"
+		"Copyright (c) 1999-2006 Scott Scriven, William Ferrell, and misc. contributors.\n"
+		"This program is released under the terms of the GNU General Public License.\n"
+		"\n"
+		"Usage: lcdproc [<options>] [<screens> ...]\n"
 		"  where <options> are\n"
-		"    -s <host>               connect to LCDd daemon on <host>\n"
-		"    -p <port>               connect to LCDd daemon using <port>\n"
-		"    -d                      daemonize\n"
-		"    -e <delay>              slow down initial announcement of modes (in 1/100s)\n"
-		"    -h                      show this help screen\n"
-		"    -v                      display program version\n"
-		"  and <modes> are\n"
-		"    C CPU                   detailed CPU usage\n"
-		"    P SMP-CPU               CPU usage overview (one line per CPU)\n"
-		"    G CPUGraph              CPU usage histogram\n"
-		"    L Load                  load histogram\n"
-		"    M Memory                memory & swap usage\n"
-		"    S ProcSize              biggest processes size\n"
-		"    D Disk                  disk usage\n"
-		"    B Battery               battery status\n"
-		"    T TimeDate              time & date information\n"
-		"    O OldTime               old time screen\n"
-		"    U Uptime                old uptime screen\n"
-		"    K BigClock              big clock\n"
-		"    N MiniClock             minimal clock\n"
-		"    A About                 credits page\n"
+		"    -s <host>           connect to LCDd daemon on <host>\n"
+		"    -p <port>           connect to LCDd daemon using <port>\n"
+		"    -f                  run in foreground\n"
+		"    -e <delay>          slow down initial announcement of screens (in 1/100s)\n"
+		"    -c <config>         use a configuration file other than %s\n"
+		"    -h                  show this help screen\n"
+		"    -v                  display program version\n"
+		"  and <screens> are\n"
+		"    C CPU               detailed CPU usage\n"
+		"    P SMP-CPU           CPU usage overview (one line per CPU)\n"
+		"    G CPUGraph          CPU usage histogram\n"
+		"    L Load              load histogram\n"
+		"    M Memory            memory & swap usage\n"
+		"    S ProcSize          biggest processes size\n"
+		"    D Disk              filling level of mounted file systems\n"
+		"    I Iface             network interface usage\n"
+		"    B Battery           battery status\n"
+		"    T TimeDate          time & date information\n"
+		"    O OldTime           old time screen\n"
+		"    U Uptime            uptime screen\n"
+		"    K BigClock          big clock\n"
+		"    N MiniClock         minimal clock\n"
+		"    A About             credits page\n"
 		"\n"
 		"Example:\n"
-		"    lcdproc -s my.lcdproc.server.com p 13666 C M X\n"
-		);
+		"    lcdproc -s my.lcdproc.server.com -p 13666 C M L\n"
+		, DEFAULT_CONFIGFILE);
 
 	exit(exit_state);
 }
@@ -291,6 +445,12 @@ HelpScreen (int exit_state)
 void
 exit_program(int val)
 {
+#ifdef LCDPROC_EYEBOXONE
+	/*
+	 * Clear Eyebox Leds
+	 */
+	eyebox_clear();
+#endif
 	//printf("exit program\n");
 	Quit = 1;
 	sock_close(sock);
@@ -374,7 +534,7 @@ main_loop()
 			//for (i=0; i<argc; i++) argv[i]=NULL; // Get rid of old tokens
 			argc = 0;
 			newtoken = 1;
-			
+
 			for (i = 0; i < len; i++) {
 				switch (buf[i]) {
 					case ' ':
@@ -433,7 +593,10 @@ main_loop()
 										lcd_cellhgt = atoi(argv[++a]);
 								}
 								connected = 1;
-								sock_printf(sock, "client_set -name {LCDproc %s}\n", get_hostname());
+								if (displayname != NULL)
+									sock_printf(sock, "client_set -name \"%s\"\n", displayname);
+								else
+									sock_printf(sock, "client_set -name {LCDproc %s}\n", get_hostname());
 #ifdef LCDPROC_MENUS
 								menus_init();
 #endif
