@@ -17,12 +17,12 @@
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
-*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+*  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *  ---
 * 
 *  $Source: /cvsroot/lcdproc/lcdproc/clients/lcdproc/util.h,v $
-*  $Revision: 1.1 $
+*  $Revision: 1.1.2.2 $
 *  Checked in by: $Author: marschap $
 *
 *******************************************************************************/
@@ -40,6 +40,9 @@ char *sprintf_memory(char *dst, double value, double roundlimit);
 /** print a percentage value to a given string */
 char *sprintf_percent(char *dst, double percent);
 
+/** converts value into power-of-x representation */
+char *convert_double(double *value, int base, double roundlimit);
+
 #endif /* LCDPROC_UTIL_H */
 
 /* EOF */
