@@ -1,12 +1,12 @@
 /*  This is the LCDproc driver for IO-Warrior devices (http://www.codemercs.de)
 
-      Copyright(C) 2004,2005 Peter Marschall <peter@adpm.de>
+      Copyright(C) 2004-2006 Peter Marschall <peter@adpm.de>
 
    based on GPL'ed code:
 
    * IOWarrior LCD routines 
        Copyright (c) 2004  Christian Vogelgsang <chris@lallafa.de>
-	
+
    * misc. files from LCDproc source tree
 
   This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 */
 
 
 #include <stdlib.h>
@@ -44,6 +44,7 @@
 #include "IOWarrior.h"
 #include "report.h"
 #include "lcd_lib.h"
+#include "adv_bignum.h"
 
 
 
@@ -74,7 +75,7 @@ static int iow_led_wcmd(usb_dev_handle *udh,int len,unsigned char *data)
 /* ------------------- IOWarrior LCD routines ------------------------------ */
 
 /* start IOWarrior's LCD mode */
-int iowlcd_enable(usb_dev_handle *udh)
+static int iowlcd_enable(usb_dev_handle *udh)
 {
   unsigned char lcd_cmd[8] = { 0x04, 0x01, 0, 0, 0, 0, 0, 0 };
   int res = iow_lcd_wcmd(udh,lcd_cmd);
@@ -83,7 +84,7 @@ int iowlcd_enable(usb_dev_handle *udh)
 }
 
 /* leave IOWarrior's LCD mode */
-int iowlcd_disable(usb_dev_handle *udh)
+static int iowlcd_disable(usb_dev_handle *udh)
 {
   unsigned char lcd_cmd[8] = { 0x04, 0x00, 0, 0, 0, 0, 0, 0 };
   int res = iow_lcd_wcmd(udh,lcd_cmd);
@@ -92,7 +93,7 @@ int iowlcd_disable(usb_dev_handle *udh)
 }
 
 /* clear IOWarrior's display */
-int iowlcd_display_clear(usb_dev_handle *udh)
+static int iowlcd_display_clear(usb_dev_handle *udh)
 {
   unsigned char lcd_cmd[8] = { 0x05, 1, 0x01, 0, 0, 0, 0, 0 };
   int res = iow_lcd_wcmd(udh,lcd_cmd);
@@ -100,7 +101,7 @@ int iowlcd_display_clear(usb_dev_handle *udh)
   return res;
 }
 
-int iowlcd_display_on_off(usb_dev_handle *udh,int display,int cursor,int blink)
+static int iowlcd_display_on_off(usb_dev_handle *udh,int display,int cursor,int blink)
 {
   unsigned char lcd_cmd[8] = { 0x05, 1, 0x08, 0, 0, 0, 0, 0 };
   if (display) lcd_cmd[2] |= 0x04;
@@ -109,7 +110,7 @@ int iowlcd_display_on_off(usb_dev_handle *udh,int display,int cursor,int blink)
   return iow_lcd_wcmd(udh,lcd_cmd);
 }
 
-int iowlcd_set_function(usb_dev_handle *udh,int eight_bit,int two_line,int ten_dots)
+static int iowlcd_set_function(usb_dev_handle *udh,int eight_bit,int two_line,int ten_dots)
 {
   unsigned char lcd_cmd[8] = { 0x05, 1, 0x20, 0, 0, 0, 0, 0 };
   if (eight_bit) lcd_cmd[2] |= 0x10;
@@ -118,21 +119,21 @@ int iowlcd_set_function(usb_dev_handle *udh,int eight_bit,int two_line,int ten_d
   return iow_lcd_wcmd(udh,lcd_cmd);
 }
 
-int iowlcd_set_cgram_addr(usb_dev_handle *udh,int addr)
+static int iowlcd_set_cgram_addr(usb_dev_handle *udh,int addr)
 {
   unsigned char lcd_cmd[8] = { 0x05, 1, 0x40, 0, 0, 0, 0, 0 };
   lcd_cmd[2] |= (addr & 0x3f);
   return iow_lcd_wcmd(udh,lcd_cmd);
 }
 
-int iowlcd_set_ddram_addr(usb_dev_handle *udh,int addr)
+static int iowlcd_set_ddram_addr(usb_dev_handle *udh,int addr)
 {
   unsigned char lcd_cmd[8] = { 0x05, 1, 0x80, 0, 0, 0, 0, 0 };
   lcd_cmd[2] |= (addr & 0x7f);
   return iow_lcd_wcmd(udh,lcd_cmd);
 }
 
-int iowlcd_write_data(usb_dev_handle *udh,int len,unsigned char *data)
+static int iowlcd_write_data(usb_dev_handle *udh,int len,unsigned char *data)
 {
   unsigned char lcd_cmd[8] = { 0x05, 0x80, 0, 0, 0, 0, 0, 0 };
   unsigned char *ptr = data;
@@ -160,21 +161,21 @@ int iowlcd_write_data(usb_dev_handle *udh,int len,unsigned char *data)
   return len;
 }
 
-int iowlcd_set_pos(usb_dev_handle *udh,int x,int y)
+static int iowlcd_set_pos(usb_dev_handle *udh,int x,int y)
 {
   const unsigned char lineOff[4] = { 0x00, 0x40, 0x14, 0x54 };
   unsigned char addr = lineOff[y] + x;
   return iowlcd_set_ddram_addr(udh, addr);
 }
 
-int iowlcd_set_text(usb_dev_handle *udh,int x,int y,int len,unsigned char *data)
+static int iowlcd_set_text(usb_dev_handle *udh,int x,int y,int len,unsigned char *data)
 {
   if (iowlcd_set_pos(udh,x,y) == IOW_ERROR)
     return IOW_ERROR;
   return iowlcd_write_data(udh,len,data);
 }
 
-int iowlcd_load_chars(usb_dev_handle *udh,int offset,int num,unsigned char *bits)
+static int iowlcd_load_chars(usb_dev_handle *udh,int offset,int num,unsigned char *bits)
 {
   if (iowlcd_set_cgram_addr(udh, offset<<3) == IOW_ERROR)
     return IOW_ERROR;
@@ -184,7 +185,7 @@ int iowlcd_load_chars(usb_dev_handle *udh,int offset,int num,unsigned char *bits
 
 /* ------------------- IOWarrior LED routines ------------------------------ */
 
-int iowled_on_off(usb_dev_handle *udh,int type, unsigned int pattern)
+static int iowled_on_off(usb_dev_handle *udh,int type, unsigned int pattern)
 {
   unsigned char led_cmd[4] = { 0x00, 0x00, 0x00, 0x00 };
   int i;
@@ -195,7 +196,7 @@ int iowled_on_off(usb_dev_handle *udh,int type, unsigned int pattern)
     led_cmd[i] = (unsigned char) (0xFF & pattern);
     pattern >>= 8;
   }  
-  
+
   return iow_led_wcmd(udh, (type == iowProd40) ? 4 : 2, led_cmd);
 }
 
@@ -205,8 +206,10 @@ int iowled_on_off(usb_dev_handle *udh,int type, unsigned int pattern)
  * Here start the API function
  */
 
-/*****************************************************
- * API: Open USB device and initialize it ...
+/**
+ * Initialize the driver.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Information of success (0) or failure (non-0).
  */
 MODULE_EXPORT int
 IOWarrior_init(Driver *drvthis)
@@ -216,7 +219,7 @@ char size[LCD_MAX_WIDTH+1] = DEFAULT_SIZE;
 
 struct usb_bus *busses;
 struct usb_bus *bus;
-  
+
 int w;
 int h;
 
@@ -264,7 +267,7 @@ PrivateData *p;
 
   /* special option lastline (some displays need it against the underline effect) */
   p->lastline = drvthis->config_get_bool(drvthis->name, "lastline", 0, 1);
-  
+
   /* Contrast of the LCD can be changed by adjusting a trimpot */
 
   /* End of config file parsing */
@@ -305,7 +308,8 @@ PrivateData *p;
       /* Check if this device is a Code Mercenaries IO-Warrior */
       if ((dev->descriptor.idVendor == iowVendor) &&
          ((dev->descriptor.idProduct == iowProd24) ||
-          (dev->descriptor.idProduct == iowProd40))) {
+          (dev->descriptor.idProduct == iowProd40) ||
+          (dev->descriptor.idProduct == iowProd56))) {
 
         /* IO-Warrior found; try to find it's description and serial number */
         p->udh = usb_open(dev);
@@ -409,8 +413,10 @@ PrivateData *p;
   return 0;
 }
 
-/******************************************************
- * API: Clean-up
+
+/**
+ * Close the driver (do necessary clean-up).
+ * \param drvthis  Pointer to driver structure.
  */
 MODULE_EXPORT void
 IOWarrior_close(Driver *drvthis)
@@ -445,8 +451,11 @@ PrivateData *p = drvthis->private_data;
   debug(RPT_DEBUG, "%s: closed", drvthis->name);
 }
 
-/******************************************************
- * API: Returns the display's width
+
+/**
+ * Return the display width in characters.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Number of characters the display is wide.
  */
 MODULE_EXPORT int 
 IOWarrior_width(Driver *drvthis)
@@ -458,8 +467,11 @@ PrivateData *p = drvthis->private_data;
   return p->width;
 }
 
-/******************************************************
- * API: Returns the display's height
+
+/**
+ * Return the display height in characters.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Number of characters the display is high.
  */
 MODULE_EXPORT int 
 IOWarrior_height(Driver *drvthis)
@@ -472,14 +484,47 @@ PrivateData *p = drvthis->private_data;
 }
 
 
-/******************************************************
- * API: Send what we have to the hardware
+/**
+ * Return the width of a character in pixels.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Number of pixel columns a character cell is wide.
+ */
+MODULE_EXPORT int 
+IOWarrior_cellwidth(Driver *drvthis)
+{
+PrivateData *p = drvthis->private_data;
+
+  debug(RPT_DEBUG, "%s: returning cellwidth", drvthis->name);
+
+  return p->cellwidth;
+}
+
+
+/**
+ * Return the height of a character in pixels.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Number of pixel lines a character cell is high.
+ */
+MODULE_EXPORT int 
+IOWarrior_cellheight(Driver *drvthis)
+{
+PrivateData *p = drvthis->private_data;
+
+  debug(RPT_DEBUG, "%s: returning cellheight", drvthis->name);
+
+  return p->cellheight;
+}
+
+
+/**
+ * Flush data on screen to the LCD.
+ * \param drvthis  Pointer to driver structure.
  */
 MODULE_EXPORT void
 IOWarrior_flush(Driver *drvthis)
 {
 PrivateData *p = drvthis->private_data;
-   
+
 int x, y;
 int i;
 int count;
@@ -534,8 +579,9 @@ int count;
 }
 
 
-/*********************************************************
- * API: Clears the LCD screen
+/**
+ * Clear the screen.
+ * \param drvthis  Pointer to driver structure.
  */
 MODULE_EXPORT void
 IOWarrior_clear(Driver *drvthis)
@@ -545,14 +591,18 @@ PrivateData *p = drvthis->private_data;
   memset(p->framebuf, ' ', p->width * p->height);
 
   p->ccmode = standard;
-  
+
   debug(RPT_DEBUG, "%s: cleared framebuffer", drvthis->name);
 }
 
 
-/*******************************************************************
- * API: Prints a character on the lcd display, at position (x,y).
- * The upper-left is (1,1), and the lower right is (p->width,p->height).
+/**
+ * Print a character on the screen at position (x,y).
+ * The upper-left corner is (1,1), the lower-right corner is (p->width, p->height).
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column).
+ * \param y        Vertical character position (row).
+ * \param c        Character that gets written.
  */
 MODULE_EXPORT void
 IOWarrior_chr(Driver *drvthis, int x, int y, char c)
@@ -569,9 +619,14 @@ PrivateData *p = drvthis->private_data;
 		  drvthis->name, (unsigned) c, x, y);
 }
 
-/*****************************************************************
- * API: Prints a string on the lcd display, at position (x,y).
- * The upper-left is (1,1), and the lower right is (p->width,p->height).
+
+/**
+ * Print a string on the screen at position (x,y).
+ * The upper-left corner is (1,1), the lower-right corner is (p->width, p->height).
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column).
+ * \param y        Vertical character position (row).
+ * \param string   String that gets written.
  */
 MODULE_EXPORT void
 IOWarrior_string(Driver *drvthis, int x, int y, char *string)
@@ -603,9 +658,10 @@ int i;
  */
 
 
-/*********************************************************
- * API: Sets the backlight
- * the API only permit setting to off=0 and on<>0
+/**
+ * Turn the LCD backlight on or off.
+ * \param drvthis  Pointer to driver structure.
+ * \param on       New backlight status.
  */
 MODULE_EXPORT void
 IOWarrior_backlight(Driver *drvthis, int on)
@@ -616,422 +672,177 @@ PrivateData *p = drvthis->private_data;
 }
 
 
-/*********************************************************
- * NOTAPI: Inits vertical bars...
- * This was part of API in 0.4 and removed in 0.5
- */
-static void
-IOWarrior_init_vbar(Driver *drvthis)
-{
-PrivateData *p = drvthis->private_data;
-
-char a[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1 };
-char b[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-char c[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-char d[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-char e[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-char f[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-char g[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-
-  if (p->ccmode != vbar) {
-    if (p->ccmode != standard) {
-      /* Not supported(yet) */
-      report(RPT_WARNING, "%s: init_vbar: cannot combine two modes using user defined characters",
-		      drvthis->name);
-      return;
-    }
-    p->ccmode = vbar;
-
-    IOWarrior_set_char(drvthis, 1, a);
-    IOWarrior_set_char(drvthis, 2, b);
-    IOWarrior_set_char(drvthis, 3, c);
-    IOWarrior_set_char(drvthis, 4, d);
-    IOWarrior_set_char(drvthis, 5, e);
-    IOWarrior_set_char(drvthis, 6, f);
-    IOWarrior_set_char(drvthis, 7, g);
-  }
-}
-
-/*********************************************************
- * NOTAPI: Inits horizontal bars...
- * This was part of API in 0.4 and removed in 0.5
- */
-static void
-IOWarrior_init_hbar(Driver *drvthis)
-{
-PrivateData *p = drvthis->private_data;
-
-char a[CELLWIDTH*CELLHEIGHT] = {
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0,
-  1, 0, 0, 0, 0 };
-char b[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0,
-  1, 1, 0, 0, 0 };
-char c[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0,
-  1, 1, 1, 0, 0 };
-char d[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  1, 1, 1, 1, 0 };
-char e[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
-
-  if (p->ccmode != hbar) {
-    if (p->ccmode != standard) {
-      /* Not supported(yet) */
-      report(RPT_WARNING, "%s: init_hbar: cannot combine two modes using user defined characters",
-		      drvthis->name);
-      return;
-    }
-
-    p->ccmode = hbar;
-
-    IOWarrior_set_char(drvthis, 1, a);
-    IOWarrior_set_char(drvthis, 2, b);
-    IOWarrior_set_char(drvthis, 3, c);
-    IOWarrior_set_char(drvthis, 4, d);
-    IOWarrior_set_char(drvthis, 5, e);
-  }
-}
-
-
-/*************************************************************
- * API: Draws a vertical bar...
+/**
+ * Draw a vertical bar bottom-up.
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column) of the starting point.
+ * \param y        Vertical character position (row) of the starting point.
+ * \param len      Number of characters that the bar is high at 100%
+ * \param promille Current height level of the bar in promille.
+ * \param options  Options (currently unused).
  */
 MODULE_EXPORT void
 IOWarrior_vbar(Driver *drvthis, int x, int y, int len, int promille, int options)
 {
 PrivateData *p = drvthis->private_data;
 
-  IOWarrior_init_vbar(drvthis);
+  if (p->ccmode != vbar) {
+    unsigned char vBar[p->cellheight];
+    int i;
 
-  /* x and y are the start position of the bar.
-   * The bar by default grows in the 'up' direction
-   *(other direction not yet implemented).
-   * len is the number of characters that the bar is long at 100%
-   * promille is the number of promilles(0..1000) that the bar should be filled.
-   */
+    if (p->ccmode != standard) {
+      /* Not supported(yet) */
+      report(RPT_WARNING, "%s: vbar: cannot combine two modes using user-defined characters",
+		      drvthis->name);
+      return;
+    }
+    p->ccmode = vbar;
+
+    memset(vBar, 0x00, sizeof(vBar));
+
+    for (i = 1; i < p->cellheight; i++) {
+      // add pixel line per pixel line ...
+      vBar[p->cellheight - i] = 0xFF;
+      IOWarrior_set_char(drvthis, i, vBar);
+    }
+  }
 
   lib_vbar_static(drvthis, x, y, len, promille, options, p->cellheight, 0);
 }
 
 
-/*****************************************************************
- * API: Draws a horizontal bar to the right.
+/**
+ * Draw a horizontal bar to the right.
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column) of the starting point.
+ * \param y        Vertical character position (row) of the starting point.
+ * \param len      Number of characters that the bar is long at 100%
+ * \param promille Current length level of the bar in promille.
+ * \param options  Options (currently unused).
  */
 MODULE_EXPORT void
 IOWarrior_hbar(Driver *drvthis, int x, int y, int len, int promille, int options)
 {
 PrivateData *p = drvthis->private_data;
 
-  IOWarrior_init_hbar(drvthis);
+  if (p->ccmode != hbar) {
+    unsigned char hBar[p->cellheight];
+    int i;
 
-  /* x and y are the start position of the bar.
-   * The bar by default grows in the 'cwrightup' direction
-   *(other direction not yet implemented).
-   * len is the number of characters that the bar is long at 100%
-   * promille is the number of promilles(0..1000) that the bar should be filled.
-   */
+    if (p->ccmode != standard) {
+      /* Not supported(yet) */
+      report(RPT_WARNING, "%s: hbar: cannot combine two modes using user-defined characters",
+		      drvthis->name);
+      return;
+    }
+
+    p->ccmode = hbar;
+
+    for (i = 1; i <= p->cellwidth; i++) {
+      // fill pixel columns from left to right.
+      memset(hBar, 0xFF & ~((1 << (p->cellwidth - i)) - 1), sizeof(hBar));
+      IOWarrior_set_char(drvthis, i, hBar);
+    }
+  }
 
   lib_hbar_static(drvthis, x, y, len, promille, options, p->cellwidth, 0);
 }
 
 
-/*******************************************************************
- * API: Sets up for big numbers
- */
-static void
-IOWarrior_init_num(Driver *drvthis)
-{
-PrivateData *p = drvthis->private_data;
-
-char bignum_ccs[8][CELLWIDTH*CELLHEIGHT] = {
-  { 1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0 },
-
-  { 0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0 },
-
-  { 1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0 },
-
-  { 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0 },
-
-  { 1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1 },
-
-  { 0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1,
-    1, 1, 0, 1, 1 },
-
-  { 1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0,
-    1, 1, 0, 0, 0 },
-
-  { 0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 1, 1,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0 }
-};
-
-  if (p->ccmode != bignum) {
-    int i;
-
-    if (p->ccmode != standard) {
-      /* Not supported (yet) */
-      report(RPT_WARNING, "%s: init_num: cannot combine two modes using user defined characters",
-		      drvthis->name);
-      return;
-    }
-
-    p->ccmode = bignum;
-
-    for (i = 0; i < NUM_CCs; i++) {
-      IOWarrior_set_char(drvthis, i, bignum_ccs[i]);
-    }
-  }
-}
-
-
-/*******************************************************************
- * API: Writes a big number.
+/**
+ * Write a big number to the screen.
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column).
+ * \param num      Character to write (0 - 10 with 10 representing ':')
  */
 MODULE_EXPORT void
 IOWarrior_num(Driver *drvthis, int x, int num)
 {
 PrivateData *p = drvthis->private_data;
+int do_init = 0;
 
-/* each bignum is constructed in a 3 x 4 matrix and consists
- * of only the 8 characters defined above as well as ' '
- *
- * The following table defines the 11 big numbers '0'-'9', ':'
- * and the custom base characters they consist of
- */
-char bignum_map[11][4][3] = {
-  { /* 0: */
-    {  1,  2,  3 },
-    {  6, 32,  6 },
-    {  6, 32,  6 },
-    {  7,  2, 32 } },
-  { /* 1: */
-    {  7,  6, 32 },
-    { 32,  6, 32 },
-    { 32,  6, 32 },
-    {  7,  2, 32 } },
-  { /* 2: */
-    {  1,  2,  3 },
-    { 32,  5,  0 },
-    {  1, 32, 32 },
-    {  2,  2,  0 } },
-  { /* 3: */
-    {  1,  2,  3 },
-    { 32,  5,  0 },
-    {  3, 32,  6 },
-    {  7,  2, 32 } },
-  { /* 4: */
-    { 32,  3,  6 },
-    {  1, 32,  6 },
-    {  2,  2,  6 },
-    { 32, 32,  0 } },
-  { /* 5: */
-    {  1,  2,  0 },
-    {  2,  2,  3 },
-    {  3, 32,  6 },
-    {  7,  2, 32 } },
-  { /* 6: */
-    {  1,  2, 32 },
-    {  6,  5, 32 },
-    {  6, 32,  6 },
-    {  7,  2, 32 } },
-  { /* 7: */
-    {  2,  2,  6 },
-    { 32,  1, 32 },
-    { 32,  6, 32 },
-    { 32,  0, 32 } },
-  { /* 8: */
-    {  1,  2,  3 },
-    {  4,  5,  0 },
-    {  6, 32,  6 },
-    {  7,  2, 32 } },
-  { /* 9: */
-    {  1,  2,  3 },
-    {  4,  3,  6 },
-    { 32,  1, 32 },
-    {  7, 32, 32 } },
-  { /* colon: (only 1st column used) */
-    { 32, 32, 32 },
-    {  0, 32, 32 },
-    {  0, 32, 32 },
-    { 32, 32, 32 } }
-};
+	if ((num < 0) || (num > 10))
+		return;
 
-  if ((num < 0) || (num > 10))
-    return;
+	if (p->ccmode != bignum) {
+		if (p->ccmode != standard) {
+			/* Not supported (yet) */
+			report(RPT_WARNING, "%s: num: cannot combine two modes using user-defined characters",
+					drvthis->name);
+			return;
+		}
 
-  IOWarrior_init_num(drvthis);
+		p->ccmode = bignum;
 
-  if (p->height >= 4) {
-    int y = (p->height - 2) / 2;	/* center vertically */
-    int x2, y2;
+		do_init = 1;
+	}
 
-    for (x2 = 0; x2 < 3; x2++) {
-      for (y2 = 0; y2 < 4; y2++) {
-        IOWarrior_chr(drvthis, x+x2, y+y2, bignum_map[num][y2][x2]);
-      }
-      if (num == 10)
-        x2 = 2; /* = break, for colon only */
-    }
-  }
-  else
-    IOWarrior_chr(drvthis, x, 1 + (p->height - 1)/ 2,
-		  (num == 10) ? ':' : (num + '0'));
+	// Lib_adv_bignum does everything needed to show the bignumbers.
+	lib_adv_bignum(drvthis, x, num, do_init, NUM_CCs);
 }
 
 
-/*********************************************************************
- * API: Sets a custom character from 0 - (NUM_CCs-1)
- *
- * The API only permit setting to off=0 and on<>0
- * For input is just an array of characters:
- *  values > 0 mean "on" and values <= 0 are "off".
+/**
+ * Set cursor position and state.
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal cursor position (column).
+ * \param y        Vertical cursor position (row).
+ * \param state    New cursor state.
  */
+/* not yet tested (needs input)
+ * maybe better: set only variables here and update when flushing
 MODULE_EXPORT void
-IOWarrior_set_char(Driver *drvthis, int n, char *dat)
+IOWarrior_cursor (Driver *drvthis, int x, int y, int state)
 {
 PrivateData *p = drvthis->private_data;
 
+  iowlcd_set_pos(p->udh, x, y);
+
+  switch (state) {
+    case CURSOR_OFF:
+      iowlcd_display_on_off(p->udh, 1, 0, 0);
+      break;
+    case CURSOR_UNDER:
+      iowlcd_display_on_off(p->udh, 1, 1, 0);
+      break;
+    case CURSOR_BLOCK:
+    case CURSOR_DEFAULT_ON:
+    default:
+      iowlcd_display_on_off(p->udh, 1, 1, 1);
+      break;
+  }    
+}
+*/
+
+
+/**
+ * Get number of custom chars available.
+ * \param drvthis  Pointer to driver structure.
+ * \returns  Number of custom characters (always NUM_CCs).
+ */
+MODULE_EXPORT int
+IOWarrior_get_free_chars (Driver *drvthis)
+{
+//PrivateData *p = drvthis->private_data;
+
+  return NUM_CCs;
+}
+
+
+/**
+ * Define a custom character and write it to the LCD.
+ * \param drvthis  Pointer to driver structure.
+ * \param n        Custom character to define [0 - (NUM_CCs-1)].
+ * \param dat      Array of 8(=cellheight) bytes, each representing a pixel row
+ *                 starting from the top to bottom.
+ *                 The bits in each byte represent the pixels where the LSB
+ *                 (least significant bit) is the rightmost pixel in each pixel row.
+ */
+MODULE_EXPORT void
+IOWarrior_set_char(Driver *drvthis, int n, unsigned char *dat)
+{
+PrivateData *p = drvthis->private_data;
+unsigned char mask = (1 << p->cellwidth) - 1;
 int row;
 
   if ((n < 0) || (n >= NUM_CCs))
@@ -1041,14 +852,10 @@ int row;
 
   for (row = 0; row < p->cellheight; row++) {
     int letter = 0;
-    int col;
 
-    if (p->lastline || (row < p->cellheight - 1)) {
-      for (col = 0; col < p->cellwidth; col++) {
-        letter <<= 1;
-        letter |= (dat[(row * p->cellwidth) + col] > 0) ? 1 : 0;
-      }
-    }  
+    if (p->lastline || (row < p->cellheight - 1))
+      letter = dat[row] & mask;	
+
     if (p->cc[n].cache[row] != letter)
       p->cc[n].clean = 0;	 /* only mark dirty if really different */
     p->cc[n].cache[row] = letter;
@@ -1056,113 +863,138 @@ int row;
 }
 
 
-/*********************************************************************
- * API: Places an icon on screen
+/**
+ * Place an icon on the screen.
+ * \param drvthis  Pointer to driver structure.
+ * \param x        Horizontal character position (column).
+ * \param y        Vertical character position (row).
+ * \param icon     synbolic value representing the icon.
+ * \return  Information whether the icon is handled here or needs to be handled by the server core.
  */
 MODULE_EXPORT int 
 IOWarrior_icon(Driver *drvthis, int x, int y, int icon)
 {
-char heart_open[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 1, 1,
-  1, 0, 1, 0, 1,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 0, 0, 0, 1,
-  1, 1, 0, 1, 1,
-  1, 1, 1, 1, 1 };
-char heart_filled[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 1, 1,
-  1, 0, 1, 0, 1,
-  0, 1, 0, 1, 0,
-  0, 1, 1, 1, 0,
-  0, 1, 1, 1, 0,
-  1, 0, 1, 0, 1,
-  1, 1, 0, 1, 1,
-  1, 1, 1, 1, 1 };
-char arrow_up[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 1, 0, 0,
-  0, 1, 1, 1, 0,
-  1, 0, 1, 0, 1,
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 0, 0, 0 };
-char arrow_down[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  1, 0, 1, 0, 1,
-  0, 1, 1, 1, 0,
-  0, 0, 1, 0, 0,
-  0, 0, 0, 0, 0 };
-char checkbox_off[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 0, 0, 0, 1,
-  1, 0, 0, 0, 1,
-  1, 0, 0, 0, 1,
-  1, 1, 1, 1, 1,
-  0, 0, 0, 0, 0 };
-char checkbox_on[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 1, 0, 0,
-  0, 0, 1, 0, 0,
-  1, 1, 1, 0, 1,
-  1, 0, 1, 1, 0,
-  1, 0, 1, 0, 1,
-  1, 0, 0, 0, 1,
-  1, 1, 1, 1, 1,
-  0, 0, 0, 0, 0 };
-char checkbox_gray[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1,
-  1, 0, 1, 0, 1,
-  1, 1, 0, 1, 1,
-  1, 0, 1, 0, 1,
-  1, 1, 1, 1, 1,
-  0, 0, 0, 0, 0 };
-char block_filled[CELLWIDTH*CELLHEIGHT] = {
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1 };
+static unsigned char heart_open[] = 
+	{ b__XXXXX,
+	  b__X_X_X,
+	  b_______,
+	  b_______,
+	  b_______,
+	  b__X___X,
+	  b__XX_XX,
+	  b__XXXXX };
+static unsigned char heart_filled[] = 
+	{ b__XXXXX,
+	  b__X_X_X,
+	  b___X_X_,
+	  b___XXX_,
+	  b___XXX_,
+	  b__X_X_X,
+	  b__XX_XX,
+	  b__XXXXX };
+static unsigned char arrow_up[] = 
+	{ b____X__,
+	  b___XXX_,
+	  b__X_X_X,
+	  b____X__,
+	  b____X__,
+	  b____X__,
+	  b____X__,
+	  b_______ };
+static unsigned char arrow_down[] = 
+	{ b____X__,
+	  b____X__,
+	  b____X__,
+	  b____X__,
+	  b__X_X_X,
+	  b___XXX_,
+	  b____X__,
+	  b_______ };
 /*
-char selector_left[CELLWIDTH*CELLHEIGHT] = {
-  0, 1, 0, 0, 0,
-  0, 1, 1, 0, 0,
-  0, 1, 1, 1, 0,
-  0, 1, 1, 1, 1,
-  0, 1, 1, 1, 0,
-  0, 1, 1, 0, 0,
-  0, 1, 0, 0, 0,
-  0, 0, 0, 0, 0 };
-char selector_right[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 1, 0,
-  0, 0, 1, 1, 0,
-  0, 1, 1, 1, 0,
-  1, 1, 1, 1, 0,
-  0, 1, 1, 1, 0,
-  0, 0, 1, 1, 0,
-  0, 0, 0, 1, 0,
-  0, 0, 0, 0, 0 };
-char ellipsis[CELLWIDTH*CELLHEIGHT] = {
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0,
-  1, 0, 1, 0, 1,
-  0, 0, 0, 0, 0 };
+static unsigned char arrow_left[] = 
+	{ b_______,
+	  b____X__,
+	  b___X___,
+	  b__XXXXX,
+	  b___X___,
+	  b____X__,
+	  b_______,
+	  b_______ };
+static unsigned char arrow_right[] = 
+	{ b_______,
+	  b____X__,
+	  b_____X_,
+	  b__XXXXX,
+	  b_____X_,
+	  b____X__,
+	  b_______,
+	  b_______ };
 */
+static unsigned char checkbox_off[] = 
+	{ b_______,
+	  b_______,
+	  b__XXXXX,
+	  b__X___X,
+	  b__X___X,
+	  b__X___X,
+	  b__XXXXX,
+	  b_______ };
+static unsigned char checkbox_on[] = 
+	{ b____X__,
+	  b____X__,
+	  b__XXX_X,
+	  b__X_XX_,
+	  b__X_X_X,
+	  b__X___X,
+	  b__XXXXX,
+	  b_______ };
+static unsigned char checkbox_gray[] = 
+	{ b_______,
+	  b_______,
+	  b__XXXXX,
+	  b__X_X_X,
+	  b__XX_XX,
+	  b__X_X_X,
+	  b__XXXXX,
+	  b_______ };
+/*
+static unsigned char selector_left[] = 
+	{ b___X___,
+	  b___XX__,
+	  b___XXX_,
+	  b___XXXX,
+	  b___XXX_,
+	  b___XX__,
+	  b___X___,
+	  b_______ };
+static unsigned char selector_right[] = 
+	{ b_____X_,
+	  b____XX_,
+	  b___XXX_,
+	  b__XXXX_,
+	  b___XXX_,
+	  b____XX_,
+	  b_____X_,
+	  b_______ };
+static unsigned char ellipsis[] = 
+	{ b_______,
+	  b_______,
+	  b_______,
+	  b_______,
+	  b_______,
+	  b_______,
+	  b__X_X_X,
+	  b_______ };
+*/	  
+static unsigned char block_filled[] = 
+	{ b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX,
+	  b__XXXXX };
 
   /* Yes we know, this is a VERY BAD implementation */
   switch (icon) {
@@ -1211,11 +1043,13 @@ char ellipsis[CELLWIDTH*CELLHEIGHT] = {
 }
 
 
-/*********************************************************************
- * API: Places an icon on screen
+/**
+ * Set output port.
+ * \param drvthis  Pointer to driver structure.
+ * \param state    Integer with bits representingthe new states
  */
 MODULE_EXPORT void
-IOWarrior_output(Driver *drvthis, int on)
+IOWarrior_output(Driver *drvthis, int state)
 {
   PrivateData *p = drvthis->private_data;
 
@@ -1223,20 +1057,22 @@ IOWarrior_output(Driver *drvthis, int on)
   if (!p->output_mask)
      return;
 
-  p->output_state = on;
-  
-  iowled_on_off(p->udh, p->productID, (on) ? p->output_mask : 0);
+  p->output_state = state;
+
+  iowled_on_off(p->udh, p->productID, state & p->output_mask);
 }
 
 
-/*********************************************************************
- * API: provides some info about this driver
+/**
+ * Provide some information about this driver.
+ * \param drvthis  Pointer to driver structure.
+ * \return  Constant string with information.
  */
 MODULE_EXPORT const char *
 IOWarrior_get_info (Driver *drvthis)
 {
   PrivateData *p = drvthis->private_data;
-  
+
   snprintf(p->info, sizeof(p->info)-1, "IOWarrior Driver: %s %s (0x%0x) S/N: %s",
            p->manufacturer, p->product, p->productID, p->serial);
   return p->info;
