@@ -16,14 +16,14 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 */
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 */
 
 /*Configfile support added by Rene Wagner (C) 2001*/
 
 /*
 Different implementations of (n)curses available on:
 OpenBSD:
- 	http://www.openbsd.org/cgi-bin/cvsweb/src/lib/libcurses/
+	http://www.openbsd.org/cgi-bin/cvsweb/src/lib/libcurses/
 	ncurses
 NetBSD:
 	http://cvsweb.netbsd.org/bsdweb.cgi/basesrc/lib/libcurses/
@@ -112,16 +112,16 @@ SunOS (5.5.1):
 
 typedef struct driver_private_data {
 	WINDOW *win;
-	
+
 	int current_color_pair;
 	int current_border_pair;
 	int curses_backlight_state;
-	
+
 	int width;
 	int height;
 	int cellwidth;
 	int cellheight;
-	
+
 	int xoffs;
 	int yoffs;
 
@@ -225,7 +225,7 @@ curses_init (Driver *drvthis)
 	buf[sizeof(buf)-1] = '\0';
 	fore_color = set_foreground_color(buf);
 	debug(RPT_DEBUG, "%s: using foreground color %s", drvthis->name, buf);
-	
+
 	/* background color */
 	strncpy(buf, drvthis->config_get_string(drvthis->name, "Background", 0, CONF_DEF_BACKGR), sizeof(buf));
 	buf[sizeof(buf)-1] = '\0';
@@ -346,11 +346,11 @@ MODULE_EXPORT void
 curses_close (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
-	
+
 	// Note that the program leaves a screen on
 	// the display to be left behind after closing;
 	// so don't clear...
-	
+
 	if (p != NULL) {
 		// Close curses
 		wrefresh(p->win);
