@@ -109,7 +109,7 @@ glk_init(Driver *drvthis)
 
   /* What speed to use */
   p->speed = drvthis->config_get_int(drvthis->name, "Speed", 0, 19200);
-  
+
   if (p->speed == 9600)       p->speed = B9600;
   else if (p->speed == 19200) p->speed = B19200;
   else if (p->speed == 38400) p->speed = B38400;
@@ -332,7 +332,7 @@ glk_flush(Driver *drvthis)
 
   for (y = 0; y < p->height; ++y) {
     int xs = -1;  /* XStart not set */
-    
+
     for (x = 0; x < p->width; ++x) {
       if ((*qf == *pf) && (xs >= 0)) {
         /* Write accumulated string */
@@ -514,26 +514,6 @@ glk_init_hbar(Driver *drvthis)
   debug(RPT_DEBUG, "glk_init_hbar()");
 }
 
-//////////////////////////////////////////////////////////////////////
-// Tells the driver to get ready for big numbers, if possible.
-//
-static void
-glk_init_num(Driver *drvthis)
-{
-  PrivateData *p = drvthis->private_data;
-
-  debug(RPT_DEBUG, "glk_init_num()");
-  if (p->fontselected != 3) {
-    /* Select Big Numbers font */
-    glkputl(p->PortFD, GLKCommand, 0x31, 3, EOF);
-    p->fontselected = 3;
-    /* Set font metrics */
-    glkputl(p->PortFD, GLKCommand, 0x32, 1, 0, 1, 1, 32, EOF);
-    /* Clear the screen */
-    glk_clear_forced(drvthis);
-  }
-}
-
 //////////////////////////////////////////////////////////////////////
 // Draws a big (4-row) number.
 //
@@ -547,6 +527,17 @@ glk_num(Driver *drvthis, int x, int num)
   if ((num < 0) || (num > 10))
       return;
 
+  if (p->fontselected != 3) {
+    debug(RPT_DEBUG, "Switching to font 3");
+    /* Select Big Numbers font */
+    glkputl(p->PortFD, GLKCommand, 0x31, 3, EOF);
+    p->fontselected = 3;
+    /* Set font metrics */
+    glkputl(p->PortFD, GLKCommand, 0x32, 1, 0, 1, 1, 32, EOF);
+    /* Clear the screen */
+    glk_clear_forced(drvthis);
+  }
+
   if ((x > 0) && (x <= p->width))
     p->framebuf[x-1] = (num >= 10) ? ':' : (num + '0');
 }
@@ -581,7 +572,7 @@ glk_old_vbar(Driver *drvthis, int x, int len)
 
   if (y >= 0) {
     int lastc;
-    
+
     switch (len) {
       case 0 :  return; break;  /* Don't output a char */
       case 1 :  lastc = 138; break;  /* One bar */
@@ -605,7 +596,7 @@ glk_old_hbar(Driver *drvthis, int x, int y, int len)
   PrivateData *p = drvthis->private_data;
 
   debug(RPT_DEBUG, "glk_old_hbar(%d, %d, %d)", x, y, len);
-  
+
   while (len > p->cellwidth) {
     glk_chr(drvthis, x, y, 255);
     ++x;
