@@ -29,11 +29,19 @@
 
 /* Constants for userdefchar_mode */
 #define NUM_CCs 8 /* number of custom characters */
-#define CCMODE_STANDARD 0 /* only char 0 is used for heartbeat */
-#define CCMODE_VBAR 1
-#define CCMODE_HBAR 2
-#define CCMODE_BIGNUM 3
-#define CCMODE_BIGCHAR 4
+
+typedef enum {
+	standard,	/* only char 0 is used for heartbeat */
+	vbar,		/* vertical bars */
+	hbar,		/* horizontal bars */
+	bignum,		/* big numbers */
+	bigchar		/* big characters */
+} CGmode;
+
+typedef struct cgram_cache {
+	unsigned char cache[LCD_DEFAULT_CELLHEIGHT];
+	int clean;
+} CGram;
 
 typedef struct ConnectionMapping {
 	char *name;
@@ -45,7 +53,11 @@ typedef struct driver_private_data {
 
 	unsigned int port;
 
-	int fd; /* for picanlcd connection type */
+	/* for serial connection type */
+	int fd;
+	int serial_type;
+
+	int charmap;
 
 	int width, height;
 	int cellwidth, cellheight;
@@ -57,8 +69,8 @@ typedef struct driver_private_data {
 	char *lcd_contents;
 
 	// The defineable characters
-	char *cc_buf;
-	char *cc_dirty;
+	CGram cc[NUM_CCs];
+	CGmode ccmode;
 
 	// Connection type data
 	int connectiontype_index;
@@ -88,7 +100,7 @@ typedef struct driver_private_data {
         char lastline;   // lastline controls the use of the last line, if pixel addressable (true, default) or
                          // underline effect (false). To avoid the underline effect, last line is always zeroed
                          // for whatever redefined character 
-    
+
 	// keyMapDirect contains an array of the ascii-codes that should be generated
 	// when a directly connected key is pressed (not in matrix).
 	char *keyMapDirect[KEYPAD_MAXX];
@@ -106,9 +118,6 @@ typedef struct driver_private_data {
 	int backlight_bit;
 
 	int output_state;	// what was most recently output to the output port
-
-	int ccmode;
-
 } PrivateData;
 
 // Structures holding pointers to HD44780 specific functions
