@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 */
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 */
 
 
 #include <stdlib.h>
@@ -37,6 +37,10 @@
 # include "config.h"
 #endif
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+#endif
+
 #include "lcd.h"
 #include "ms6931.h"
 #include "report.h"
@@ -161,7 +165,7 @@ ms6931_init (Driver *drvthis)
 	/* initialize private data */
 	p->fd = -1;
 	p->framebuf = NULL;
-	      
+
 
 	debug(RPT_INFO, "ms6931_init: init(%p)", drvthis);
 
@@ -248,7 +252,7 @@ ms6931_close (Driver *drvthis)
 			ms6931_flush(drvthis);
 			ms6931_backlight (drvthis, BACKLIGHT_OFF);
 		}	
-       
+
 		if (p->fd >= 0)
 			close(p->fd);
 
@@ -302,7 +306,7 @@ ms6931_chr (Driver *drvthis, int x, int y, char c)
 
 	y--;
 	x--;
-	
+
 	if ((x >= 0) && (y >= 0) && (x < p->width) && (y < p->height))
 		p->framebuf[(y * p->width) + x] = charTable[(unsigned char) c];
 }
@@ -390,7 +394,7 @@ ms6931_string (Driver *drvthis, int x, int y, char string[])
 
 	if ((y < 0) || (y >= p->height))
 		return;
-	
+
 	for (i = 0; (string[i] != '\0') && (x < p->width); i++, x++) {
 		unsigned char c = (unsigned char) string[i];
 
@@ -479,7 +483,7 @@ ms6931_get_key (Driver *drvthis)
 		FD_SET(p->fd, &fdset);
 		return NULL;
 	}
-	
+
 	if (!FD_ISSET(p->fd, &fdset))
 		return NULL;
 
