@@ -39,6 +39,9 @@
 #include "screen.h"
 #include "widget.h"
 
+
+extern Menu * custom_main_menu;
+
 /** Basicly a patched version of LL_GetByIndex() that ignores hidden
  * entries completely. (But it takes a menu as an argument.) */
 static void *
@@ -137,6 +140,9 @@ menu_destroy (Menu *menu)
 	if (menu == NULL)
 		return;
 
+	if (custom_main_menu == menu)
+		custom_main_menu = NULL;
+
 	menu_destroy_all_items (menu);
 	LL_Destroy (menu->data.menu.contents);
 	menu->data.menu.contents = NULL;
@@ -339,7 +345,7 @@ void menu_build_screen (MenuItem *menu, Screen *s)
 					(w->text)[display_props->width-1] = '\0';
 				}
 				break;
- 			 default:
+			 default:
 				assert(!"unexpected menuitem type");
 			}
 		}
@@ -491,7 +497,7 @@ void menu_update_screen (MenuItem *menu, Screen *s)
 		w->type = (menu->data.menu.scroll > 0) ? WID_ICON : WID_NONE;
 	else
 		report (RPT_ERR, "%s: could not find widget: %s", __FUNCTION__, "upscroller");
-	
+
 	/* Enable downscroller (if necessary) */
 	w = screen_find_widget (s, "downscroller");
 	if (w != NULL)
@@ -629,7 +635,7 @@ MenuResult menu_process_input(Menu *menu, MenuToken token, const char * key, boo
 	  case MENUTOKEN_LEFT:
 		if (!extended)
 			return MENURESULT_NONE;
-		
+
 		subitem = menu_get_subitem (menu, menu->data.menu.selector_pos);
 		if (subitem == NULL)
 			break;
@@ -661,7 +667,7 @@ MenuResult menu_process_input(Menu *menu, MenuToken token, const char * key, boo
 	  case MENUTOKEN_RIGHT:
 		if (!extended)
 			return MENURESULT_NONE;
-		
+
 		subitem = menu_get_subitem(menu, menu->data.menu.selector_pos);
 		if (subitem == NULL)
 			break;
