@@ -6,21 +6,23 @@
  * COPYING file distributed with this package.
  *
  * Copyright (c) 1999, William Ferrell, Scott Scriven
+ *               2004, F5 Networks, Inc. - IP-address verification
  *
  */
 
-#ifndef SOCKETS_H
-#define SOCKETS_H
+#ifndef SOCK_H
+#define SOCK_H
 
 #include "shared/sockets.h"
 
 typedef struct sockaddr_in sockaddr_in;
 
 /* Server functions...*/
-int sock_create_server ();
-int sock_create_inet_socket (unsigned short int port);
-int sock_poll_clients ();
-int sock_close_all ();
-int read_from_client (int filedes);
+int sock_init(char* bind_addr, int bind_port);
+int sock_shutdown();
+int sock_create_inet_socket(char* bind_addr, unsigned int port);
+int sock_poll_clients();
+int verify_ipv4(const char *addr);
+int verify_ipv6(const char *addr);
 
 #endif
